#!/usr/bin/env lua

module("fastcgi_server", package.seeall)

require"json"
require"debug"

--
-- crython globals
--

require"libcrython"
cryoboard = libcrython.CryoBoard()

backpack = {
	CryoBoard = cryoboard,
	CryoConverter = cryoboard:get_converter(),
	CryoHeaterConverter = cryoboard:get_heater_converter(),
}

function run(wsapi_env)
	local headers = { ["Content-type"] = "text/json" }

	local function response_404()
		coroutine.yield("HTTP 404! Method and object not found.\n")
	end

	-- Parse object and method
	_,_,object,method = wsapi_env.PATH_INFO:find("/([^/]*)/([^/]*)")

	if not (object and method) then
		return 404, {}, coroutine.wrap(function () coroutine.yield("HTTP 404!\n") end )
	end

	object = backpack[object]
	method = object[method]

	-- Parse arguments
	args = wsapi_env.input:read()
	args = (not args) and {} or json.decode(args)

	-- Try the call
	local returned
	wrapped = function() returned = method(object, unpack(args)) end
	ok, trace = xpcall(wrapped, debug.traceback)

	if ok then
		 cog = function()
			 coroutine.yield(json.encode({ message="ok", returned=returned }))
			 coroutine.yield("\n")
		 end
	else
		 cog = function()
			 coroutine.yield(json.encode({ error = { trace = trace } }))
			 coroutine.yield("\n")
		 end
	end

	return 200, headers, coroutine.wrap(cog)
end

local fastcgi = require"wsapi.fastcgi"
fastcgi.run(fastcgi_server.run)

