<%inherit file="base.inc" />

<h1>Readout</h1>

<script type="text/javascript">
<!--
	/*
	 * Parameter refresh functions: The following populate a single row of
	 * a table, dynamically, only reloading the configuration (as
	 * opposed to measurement) entries.
	 */
	function refresh_heater_channel(channel) {
		cryo_method('CryoHeaterConverter',
			'get_resistance', [ channel ],
			{ success: function(data) {
				$('#heater_' + channel + '_resistance').attr("innerHTML", format_si(data.returned, '&#937;'))
			} }
		);
	}

	function refresh_channel(channel) {
		cryo_method('CryoConverter', 'get_channel_name', [ channel ],
			{ success: function(data) { $('#channel_' + channel + '_name').attr("innerText", data.returned) } });
		cryo_method('CryoConverter', 'get_channel_type', [ channel ],
			{ success: function(data) { $('#channel_' + channel + '_type').attr("innerText", data.returned) } });
		cryo_method('CryoConverter', 'get_drive_resistance', [ channel ],
			{ success: function(data) { $('#channel_' + channel + '_drive_resistance').attr("innerHTML", format_si(data.returned, '&#937;')) } });
		cryo_method('CryoConverter', 'get_sense_resistance', [ channel ],
			{ success: function(data) { $('#channel_' + channel + '_sense_resistance').attr("innerHTML", format_si(data.returned, '&#937;')) } });
		cryo_method('CryoBoard', 'get_dac_amplitude', [ channel, ${self.libcrython.DAC_COUNTS} ],
			{ success: function(data) { $("#channel_" + channel + "_dac").attr("innerHTML", data.returned) } });
		cryo_method('CryoBoard', 'get_dac_amplitude', [ channel, ${self.libcrython.AMPS} ],
			{ success: function(data) { $("#channel_" + channel + "_amps").attr("innerHTML", format_si(data.returned, 'A')) } });
		cryo_method('CryoBoard', 'get_adc_waveform', [ channel ],
			{ success: function(data) { $("#channel_" + channel + "_adc_waveform").attr("innerHTML", waveforms_to_names[data.returned]); } });
		cryo_method('CryoBoard', 'get_dac_waveform', [ channel ],
			{ success: function(data) { $("#channel_" + channel + "_dac_waveform").attr("innerHTML", waveforms_to_names[data.returned]); } });
	}

	function refresh_frequency() {
		cryo_method('CryoBoard',
			'get_frequency', [ ${self.libcrython.HZ} ],
			{ success: function(data) { $("#freq_hz").attr("innerHTML", format_si(data.returned, "Hz")); } }
		);
	}

	$(document).ready(function() {

		/*
		 * Make this page editable.
		 */

		$('#freq_hz').editable(function(value, settings) {
			cryo_method('CryoBoard', 'set_frequency', [value, ${self.libcrython.HZ}], { success: function(data) { refresh_frequency() } });
		},{ submit: 'OK', style: 'inherit'});

		/* Resistances */
		$('[id$=_resistance]').editable(function(value, settings) {
			id = $(this).attr('id');

			/* Match heater resistance fields; parse their IDs */
			if(x = id.match('heater_([0-9]*)_resistance')) {
				cryo_method('CryoHeaterConverter',
					'set_resistance', [
						x[1],
						value
					],
					{ complete: function() { refresh_heater_channel(x[1])} }
				);
			} else if(x = id.match('channel_([0-9]*)_(drive|sense)_resistance')) {
				cryo_method('CryoConverter',
					'set_' + x[2] + '_resistance', [
						x[1],
						value
					],
					{ complete: function() { refresh_channel(x[1])} }
				);
			} else {
				console.log("Unknown resistance ID " + id);
			}

			return(value);
		},{ submit: 'OK', style:'inherit', placeholder:'(n/a)' });

		/* Waveform selectors */
		$('[id$=_waveform]').editable(function(value, settings) {
			id = $(this).attr('id');
			x = id.match('channel_([0-9]*)_(adc|dac)_waveform')
			cryo_method('CryoBoard','set_'+x[2]+'_waveform',
					[ x[1], names_to_waveforms[value] ],
					{ complete: function() { refresh_channel(x[1])} }
			);
		},{ data: {'AC':'AC','DC':'DC'}, type: 'select', submit: 'OK', style:'inherit' });

		/* Amplitude settings */
		$('[id$=_dac],[id$=_volts],[id$=amps],[id$=watts]').filter(':not([id*=measured])').editable(function(value, settings) {
			id = $(this).attr('id');

			x = id.match('(heater|channel)_([0-9]*)_(dac|volts|amps|watts)');

			switch(x[1]) {
				case 'heater':
					method='set_heater_amplitude';
					completion = refresh_heater_channel;
					break;
				case 'channel':
					method='set_dac_amplitude';
					completion = refresh_channel;
					break;
			}

			switch(x[3]) {
				case 'dac': units=${self.libcrython.DAC_COUNTS}; break;
				case 'volts': units=${self.libcrython.VOLTS}; break;
				case 'amps': units=${self.libcrython.AMPS}; break;
				case 'watts': units=${self.libcrython.WATTS}; break;
			}

			cryo_method('CryoBoard', method, [
					x[2],
					parseFloat(value),
					units
				],
				{ complete: function() { completion(x[2])} }
			);

			return(value);
		},{ submit: 'OK', style:'inherit', placeholder:'(n/a)' });

		/* Name */
		$('[id$=_name]').editable(function(value, settings) {
			id = $(this).attr('id');

			/* Match heater name; parse ID */
			if(x = id.match('heater_([0-9]*)_name')) {
				cryo_method('CryoHeaterConverter',
					'set_channel_name', [
						x[1],
						value
					],
					{ complete: function() { refresh_heater_channel(x[1])} }
				);
			} else if(x = id.match('channel_([0-9]*)_name')) {
				cryo_method('CryoConverter',
					'set_channel_name', [
						x[1],
						value
					],
					{ complete: function() { refresh_channel(x[1])} }
				);
			} else {
				console.log("Unknown name pattern " + id);
			}

			return(value);
		},{ submit: 'OK', style:'inherit', placeholder:'(n/a)' });

		/* Type */
		$('[id$=_type]').editable(function(value, settings) {
			id = $(this).attr('id');
			x = id.match('channel_([0-9]*)_type')
			cryo_method('CryoConverter','set_channel_type',
					[ x[1], value ],
					{ complete: function() { refresh_channel(x[1])} }
			);
		},{ data: {'Diode':'Diode','RTD':'RTD'}, type: 'select', submit: 'OK', style:'inherit' });

		/*
		 * Initialization occurs below. We kick off dump_state and
		 * dump_measurement calls and fill in the blanks when they
		 * complete.
		 */

		/* Fetch data from cryo board */
		cryo_method('CryoBoard', 'dump_state', [ null ], { success: function(data) {
			data = data.returned;

			for(var i=0; i<${self.libcrython.NUM_HEATER_CHANNELS}; i++) {
				$("#heater_" + i + "_name").attr("innerText", data.heater_channels[i].name);
				$("#heater_" + i + "_switch").attr("checked", data.heater_channels[i].state);
				$("#heater_" + i + "_resistance").attr("innerHTML", format_si(data.heater_channels[i].resistance, "&#937;"));
				$("#heater_" + i + "_dac").attr("innerHTML", data.heater_channels[i].amp[${str(self.libcrython.DAC_COUNTS)}]);
				$("#heater_" + i + "_volts").attr("innerHTML", format_si(data.heater_channels[i].amp[${str(self.libcrython.VOLTS)}], "V"));
				$("#heater_" + i + "_amps").attr("innerHTML", format_si(data.heater_channels[i].amp[${str(self.libcrython.AMPS)}], "A"));
				$("#heater_" + i + "_watts").attr("innerHTML", format_si(data.heater_channels[i].amp[${str(self.libcrython.WATTS)}], "W"));
			}

			$("#freq_hz").attr("innerHTML", format_si(data.frequency, "Hz"));

			for(var i=0; i<${self.libcrython.NUM_COOKED_CHANNELS}; i++) {
				$("#channel_" + i + "_name").attr("innerText", data.sensor_channels[i].name);
				$("#channel_" + i + "_type").attr("innerText", data.sensor_channels[i].type);
				$("#channel_" + i + "_drive_resistance").attr("innerHTML", format_si(data.sensor_channels[i].drive_resistance, "&#937;"));
				$("#channel_" + i + "_sense_resistance").attr("innerHTML", format_si(data.sensor_channels[i].sense_resistance, "&#937;"));
				$("#channel_" + i + "_dac").attr("innerHTML", data.sensor_channels[i].dac_amplitude[${str(self.libcrython.DAC_COUNTS)}]);
				$("#channel_" + i + "_amps").attr("innerHTML", format_si(data.sensor_channels[i].dac_amplitude[${str(self.libcrython.AMPS)}], "A"));
				$("#channel_" + i + "_dac_waveform").attr("innerHTML", waveforms_to_names[data.sensor_channels[i].dac_waveform]);
				$("#channel_" + i + "_adc_waveform").attr("innerHTML", waveforms_to_names[data.sensor_channels[i].adc_waveform]);
			}
		}});

		/* Fetch data from cryo board */
		cryo_method('CryoBoard', 'dump_measurements', [ null ], { success: function(data) {
			data = data.returned;

			for(var i=0; i<${self.libcrython.NUM_COOKED_CHANNELS}; i++) {
				$("#channel_" + i + "_measured_volts").attr("innerHTML", format_si(data.sensor_channels[i].adc_amplitude[${str(self.libcrython.VOLTS)}], "V"));
				$("#channel_" + i + "_measured_amps").attr("innerHTML", format_si(data.sensor_channels[i].adc_amplitude[${str(self.libcrython.AMPS)}], "A"));
				$("#channel_" + i + "_measured_resistance").attr("innerHTML", format_si(data.sensor_channels[i].adc_amplitude[${str(self.libcrython.OHMS)}], "&#937;"));
				$("#channel_" + i + "_measured_temperature").attr("innerHTML", format_si(data.sensor_channels[i].adc_amplitude[${str(self.libcrython.KELVIN)}], "K"));
			}
		}});

		/*
		 * Timed call that refills only the measurement entries
		 */
		$.timer(10000, function(timer) { cryo_method('CryoBoard', 'dump_measurements', [null], { success: function(data) {
			data = data.returned;
			for(var i=0; i<${self.libcrython.NUM_COOKED_CHANNELS}; i++) {
				$("#channel_" + i + "_measured_volts").attr("innerHTML", format_si(data.sensor_channels[i].adc_amplitude[${str(self.libcrython.VOLTS)}], "V"));
				$("#channel_" + i + "_measured_amps").attr("innerHTML", format_si(data.sensor_channels[i].adc_amplitude[${str(self.libcrython.AMPS)}], "A"));
				$("#channel_" + i + "_measured_resistance").attr("innerHTML", format_si(data.sensor_channels[i].adc_amplitude[${str(self.libcrython.OHMS)}], "&#937;"));
				$("#channel_" + i + "_measured_temperature").attr("innerHTML", format_si(data.sensor_channels[i].adc_amplitude[${str(self.libcrython.KELVIN)}], "K"));
			}
		}})});

	});
-->
</script>

<table summary='Heater Status' id="tablecol">
	<caption>Heater Status and Readout</caption>
	<thead>
		<tr>
			<th>#</th><th>Channel</th><th>Switch</th><th>Resistance</th><th>DAC Counts</th><th>Volts</th><th>Amps</th><th>Power</th>
		</tr>
	</thead>
	<tbody>
		% for c in xrange(self.libcrython.NUM_HEATER_CHANNELS):
		<tr>
			<td>${c+1}</td>
			<td id="heater_${c}_name"></td>
			<td>
				<input type="checkbox" id="heater_${c}_switch"
					onclick="cryo_method('CryoBoard', 'set_heater_switch', [
						${c},
						$('#heater_${c}_switch').attr('checked')
					])" />
			</td>
			<td id="heater_${c}_resistance" style="width:inherit;" align="right"></td>
			<td align="right" id="heater_${c}_dac"></td>
			<td align="right" id="heater_${c}_volts"></td>
			<td align="right" id="heater_${c}_amps"></td>
			<td align="right" id="heater_${c}_watts"></td>
		</tr>
		% endfor
	</tbody>
</table>

<table border= '1' summary = "Sense and drive status">
	<thead>
		<caption>Sense and Drive Settings and Readout</caption>
		<tr>
			<th colspan=13>Mod/Demod Freq: <span id="freq_hz"></span></th>
		</tr>
		<tr>
			<th>Channel Number</th>
			<th>Channel Name</th>
			<th>Sensor Type</th>
			<th>Drive Gain Resistor</th>
			<th>Sense Gain Resistor</th>
			<th>DAC Setting</th>
			<th>Current Setting</th>
			<th>Biasing Waveform</th>
			<th>Readout Waveform</th>
			<th>Measured Voltage (RMS)</th>
			<th>Measured Current (RMS)</th>
			<th>Measured Resistance</th>
			<th>Temperature</th>
		</tr>
	</thead>
	<tbody>
		% for c in xrange(self.libcrython.NUM_COOKED_CHANNELS):
		<tr>
			<td>${c+1}</td>
			<td id="channel_${c}_name"></td>
			<td id="channel_${c}_type"></td>
			<td id="channel_${c}_drive_resistance"></td>
			<td id="channel_${c}_sense_resistance"></td>
			<td id="channel_${c}_dac"></td>
			<td id="channel_${c}_amps"></td>
			<td id="channel_${c}_dac_waveform"></td>
			<td id="channel_${c}_adc_waveform"></td>
			<td id="channel_${c}_measured_volts"></td>
			<td id="channel_${c}_measured_amps"></td>
			<td id="channel_${c}_measured_resistance"></td>
			<td id="channel_${c}_measured_temperature"></td>
		</tr>
		% endfor
	</tbody>
</table>

